/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#include <glib.h>
#include <gdk/gdkx.h>
#include <cairo-glitz.h>
#include <glitz-glx.h>

#include "glitz-helper.h"

static glitz_drawable_format_t* kiba_glitz_find_draw_format () {
	glitz_drawable_format_t templ, *format, *draw_format;
	XVisualInfo *vinfo = NULL;
	unsigned long mask = 0;
	
	templ.doublebuffer = 1;
	mask |= GLITZ_FORMAT_DOUBLEBUFFER_MASK;
	
	int i = 0;
	do {
		format = glitz_glx_find_window_format (gdk_display,
						DefaultScreen (gdk_display),
						mask, &templ, i++);
		if (format) {
			vinfo = glitz_glx_get_visual_info_from_format (gdk_display,
							DefaultScreen (gdk_display),
							format);
			if (vinfo->depth == 32) {
				draw_format = format;
				break;
			}
			else {
				if (!draw_format)
					draw_format = format;
			}
		}
	} while (format);
	
	return draw_format;
}

void kiba_glitz_get_colormap (glitz_drawable_format_t **pGlitzDrawFormat,
			      GdkVisual               **visual,
			      GdkColormap             **colormap) {
	XVisualInfo *vinfo = NULL;
	
	if (!*pGlitzDrawFormat)
		*pGlitzDrawFormat = kiba_glitz_find_draw_format ();
	if (!*pGlitzDrawFormat)
		fprintf (stderr, "GLITZ - no double buffered GLX visual\n");
	else {
		vinfo = glitz_glx_get_visual_info_from_format (gdk_display,
				DefaultScreen (gdk_display),
				*pGlitzDrawFormat);
		*visual    = gdkx_visual_get (vinfo->visualid);
		*colormap  = gdk_colormap_new (*visual, TRUE);
	}
}

void kiba_glitz_create_drawable (Window                    window,
				 int                       iWidth,
				 int                       iHeight,
				 glitz_drawable_format_t  *pGlitzDrawFormat,
				 glitz_format_t          **pGlitzFormat,
				 glitz_drawable_t        **pGlitzDrawable) {
	glitz_format_t templ_f;
	
	if (!pGlitzDrawFormat) {
		fprintf (stderr , "GLITZ - No valid draw format for drawable\n");
		return;
	}
	
	*pGlitzDrawable = glitz_glx_create_drawable_for_window (gdk_display,
				DefaultScreen (gdk_display),
				pGlitzDrawFormat,
				window,
				iWidth, iHeight);
	if (!*pGlitzDrawable) {
		fprintf (stderr, "GLITZ - failed to create drawable\n");
		return;
	}
	
	templ_f.color        = pGlitzDrawFormat->color;
	templ_f.color.fourcc = GLITZ_FOURCC_RGB;
	
	*pGlitzFormat =
		glitz_find_format (*pGlitzDrawable,
				GLITZ_FORMAT_RED_SIZE_MASK   |
				GLITZ_FORMAT_GREEN_SIZE_MASK |
				GLITZ_FORMAT_BLUE_SIZE_MASK  |
				GLITZ_FORMAT_ALPHA_SIZE_MASK |
				GLITZ_FORMAT_FOURCC_MASK,
				&templ_f,
				0);
	if (!*pGlitzFormat) {
		fprintf (stderr, "GLITZ - couldn't find surface format\n");
		*pGlitzDrawable = NULL;
	}
}

cairo_t* kiba_glitz_get_cairo_context (glitz_drawable_t        *pGlitzDrawable,
				       glitz_drawable_format_t *pGlitzDrawFormat,
				       glitz_format_t          *pGlitzFormat,
				       int                      iWidth,
				       int                      iHeight) {
	cairo_t *cr = NULL;
	cairo_surface_t *cairo_surface;
	glitz_surface_t *glitz_surface;
	
	if (!pGlitzDrawable)
		return NULL;
	
	glitz_surface = glitz_surface_create (pGlitzDrawable,
	pGlitzFormat, iWidth, iHeight, 0, NULL);
	
	if (pGlitzDrawFormat->doublebuffer)
		glitz_surface_attach (glitz_surface, pGlitzDrawable,
				GLITZ_DRAWABLE_BUFFER_BACK_COLOR);
	else
		glitz_surface_attach (glitz_surface, pGlitzDrawable,
				GLITZ_DRAWABLE_BUFFER_FRONT_COLOR);
	
	cairo_surface = cairo_glitz_surface_create (glitz_surface);
	cr = cairo_create (cairo_surface);
	
	cairo_surface_destroy (cairo_surface);
	glitz_surface_destroy (glitz_surface);
	
	return cr;
}

int kiba_glitz_draw_func (glitz_drawable_t        *pGlitzDrawable,
			  glitz_drawable_format_t *pGlitzDrawFormat) {
	if (pGlitzDrawable && pGlitzDrawFormat && pGlitzDrawFormat->doublebuffer)
		glitz_drawable_swap_buffers (pGlitzDrawable);
	else if (pGlitzDrawable && pGlitzDrawFormat)
		glitz_drawable_flush (pGlitzDrawable);
	else
		return 0;
	
	return 1;
}

void kiba_glitz_resize_func (glitz_drawable_t *pGlitzDrawable,
			     int               iWidth,
			     int               iHeight) {
	if (!pGlitzDrawable)
		return;
	
	glitz_drawable_update_size (pGlitzDrawable, iWidth, iHeight);
}

void kiba_glitz_fini (glitz_drawable_t *pGlitzDrawable) {
	if (pGlitzDrawable)
		glitz_drawable_destroy (pGlitzDrawable);
}
